(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('redux')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react', 'redux'], factory) :
  (factory((global.ReactLocalRedux = {}),global.React,global.Redux));
}(this, (function (exports,react,redux) { 'use strict';

  function _objectWithoutPropertiesLoose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;

    for (i = 0; i < sourceKeys.length; i++) {
      key = sourceKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      target[key] = source[key];
    }

    return target;
  }

  /**
   * @constant {boolean} HAS_MAP_SUPPORT
   */
  var HAS_MAP_SUPPORT = typeof Map === 'function';

  /**
   * @constant {boolean} HAS_SET_SUPPORT
   */
  var HAS_SET_SUPPORT = typeof Set === 'function';

  /**
   * @constant {boolean} HAS_WEAKSET_SUPPORT
   */
  var HAS_WEAKSET_SUPPORT = typeof WeakSet === 'function';

  // constants

  var keys = Object.keys;

  /**
   * @function addObjectToCache
   *
   * @description
   * add object to cache if it is indeed an object
   *
   * @param {any} object the object to potentially add to the cache
   * @param {Object|WeakSet} cache the cache to add to
   * @returns {void}
   */
  var addObjectToCache = function addObjectToCache(object, cache) {
    return object && typeof object === 'object' && cache.add(object);
  };

  /**
   * @function hasItem
   *
   * @description
   * does the array include the item passed
   *
   * @param {Array<any>} array the array to check in
   * @param {any} item the item to locate
   * @param {function} isEqual the equality comparator
   * @param {any} meta the meta item to pass through
   * @returns {boolean} does the item exist in the array
   */
  var hasItem = function hasItem(array, item, isEqual, meta) {
    for (var index = 0; index < array.length; index++) {
      if (isEqual(array[index], item, meta)) {
        return true;
      }
    }

    return false;
  };

  /**
   * @function hasItems
   *
   * @description
   * are the arrays equal in value, despite being in different order
   *
   * @param {Array<any>} arrayA the first array to test
   * @param {Array<any>} arrayB the second array to test
   * @param {function} isEqual the equality comparator
   * @param {any} meta the meta item to pass through
   * @returns {boolean} are the arrays equal absent order
   */
  var hasItems = function hasItems(arrayA, arrayB, isEqual, meta) {
    if (arrayA.length !== arrayB.length) {
      return false;
    }

    for (var index = 0; index < arrayA.length; index++) {
      if (!hasItem(arrayB, arrayA[index], isEqual, meta)) {
        return false;
      }
    }

    return true;
  };

  /**
   * @function sameValueZeroEqual
   *
   * @description
   * are the objects passed strictly equal or both NaN
   *
   * @param {any} objectA the object to compare against
   * @param {any} objectB the object to test
   * @returns {boolean} are the objects equal by the SameValueZero principle
   */
  var sameValueZeroEqual = function sameValueZeroEqual(objectA, objectB) {
    return objectA === objectB || objectA !== objectA && objectB !== objectB;
  };

  /**
   * @function isPlainObject
   *
   * @description
   * is the object a plain object
   *
   * @param {any} object the object to test
   * @returns {boolean} is the object a plain object
   */
  var isPlainObject = function isPlainObject(object) {
    return object.constructor === Object;
  };

  /**
   * @function isPromiseLike
   *
   * @description
   * is the object promise-like (thenable)
   *
   * @param {any} object the object to test
   * @returns {boolean} is the object promise-like
   */
  var isPromiseLike = function isPromiseLike(object) {
    return typeof object.then === 'function';
  };

  /**
   * @function isReactElement
   *
   * @description
   * is the object passed a react element
   *
   * @param {any} object the object to test
   * @returns {boolean} is the object a react element
   */
  var isReactElement = function isReactElement(object) {
    return !!(object.$$typeof && object._store);
  };

  /**
   * @function getNewCache
   *
   * @description
   * get a new cache object to prevent circular references
   *
   * @returns {Object|Weakset} the new cache object
   */
  var getNewCache = function getNewCache() {
    return HAS_WEAKSET_SUPPORT ? new WeakSet() : Object.create({
      _values: [],
      add: function add(value) {
        this._values.push(value);
      },
      has: function has(value) {
        return !!~this._values.indexOf(value);
      }
    });
  };

  /**
   * @function createCircularEqual
   *
   * @description
   * create a custom isEqual handler specific to circular objects
   *
   * @param {funtion} [isEqual] the isEqual comparator to use instead of isDeepEqual
   * @returns {function(any, any): boolean}
   */
  var createCircularEqual = function createCircularEqual(isEqual) {
    return function (isDeepEqual) {
      var comparator = isEqual || isDeepEqual;

      return function (objectA, objectB) {
        var cache = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : getNewCache();

        var cacheHasA = cache.has(objectA);
        var cacheHasB = cache.has(objectB);

        if (cacheHasA || cacheHasB) {
          return cacheHasA && cacheHasB;
        }

        addObjectToCache(objectA, cache);
        addObjectToCache(objectB, cache);

        return comparator(objectA, objectB, cache);
      };
    };
  };

  /**
   * @function toPairs
   *
   * @param {Map|Set} iterable the iterable to convert to [key, value] pairs (entries)
   * @returns {{keys: Array<*>, values: Array<*>}} the [key, value] pairs
   */
  var toPairs = function toPairs(iterable) {
    var pairs = {
      keys: [],
      values: []
    };

    iterable.forEach(function (value, key) {
      return pairs.keys.push(key) && pairs.values.push(value);
    });

    return pairs;
  };

  /**
   * @function areArraysEqual
   *
   * @description
   * are the arrays equal in value
   *
   * @param {Array<any>} arrayA the array to test
   * @param {Array<any>} arrayB the array to test against
   * @param {function} isEqual the comparator to determine equality
   * @param {any} meta the meta object to pass through
   * @returns {boolean} are the arrays equal
   */
  var areArraysEqual = function areArraysEqual(arrayA, arrayB, isEqual, meta) {
    if (arrayA.length !== arrayB.length) {
      return false;
    }

    for (var index = 0; index < arrayA.length; index++) {
      if (!isEqual(arrayA[index], arrayB[index], meta)) {
        return false;
      }
    }

    return true;
  };

  var createAreIterablesEqual = function createAreIterablesEqual(shouldCompareKeys) {
    /**
     * @function areIterablesEqual
     *
     * @description
     * determine if the iterables are equivalent in value
     *
     * @param {Array<Array<any>>} pairsA the pairs to test
     * @param {Array<Array<any>>} pairsB the pairs to test against
     * @param {function} isEqual the comparator to determine equality
     * @param {any} meta the cache possibly being used
     * @returns {boolean} are the objects equal in value
     */
    var areIterablesEqual = shouldCompareKeys ? function (pairsA, pairsB, isEqual, meta) {
      return hasItems(pairsA.keys, pairsB.keys, isEqual, meta) && hasItems(pairsA.values, pairsB.values, isEqual, meta);
    } : function (pairsA, pairsB, isEqual, meta) {
      return hasItems(pairsA.values, pairsB.values, isEqual, meta);
    };

    return function (iterableA, iterableB, isEqual, meta) {
      return areIterablesEqual(toPairs(iterableA), toPairs(iterableB), isEqual, meta);
    };
  };

  /**
   * @function areArraysEqual
   *
   * @description
   * are the objects equal in value
   *
   * @param {Array<any>} objectA the object to test
   * @param {Array<any>} objectB the object to test against
   * @param {function} isEqual the comparator to determine equality
   * @param {any} meta the meta object to pass through
   * @returns {boolean} are the objects equal
   */
  var areObjectsEqual = function areObjectsEqual(objectA, objectB, isEqual, meta) {
    var keysA = keys(objectA);
    var keysB = keys(objectB);

    if (keysA.length !== keysB.length) {
      return false;
    }

    var key = void 0;

    for (var index = 0; index < keysA.length; index++) {
      key = keysA[index];

      if (!hasItem(keysB, key, sameValueZeroEqual)) {
        return false;
      }

      // if a react element, ignore the "_owner" key because its not necessary for equality comparisons
      if (key === '_owner' && isReactElement(objectA) && isReactElement(objectB)) {
        continue;
      }

      if (!isEqual(objectA[key], objectB[key], meta)) {
        return false;
      }
    }

    return true;
  };

  /**
   * @function areRegExpsEqual
   *
   * @description
   * are the regExps equal in value
   *
   * @param {RegExp} regExpA the regExp to test
   * @param {RegExp} regExpB the regExp to test agains
   * @returns {boolean} are the regExps equal
   */
  var areRegExpsEqual = function areRegExpsEqual(regExpA, regExpB) {
    return regExpA.source === regExpB.source && regExpA.global === regExpB.global && regExpA.ignoreCase === regExpB.ignoreCase && regExpA.multiline === regExpB.multiline && regExpA.unicode === regExpB.unicode && regExpA.sticky === regExpB.sticky && regExpA.lastIndex === regExpB.lastIndex;
  };

  // constants

  var isArray = Array.isArray;

  var areMapsEqual = createAreIterablesEqual(true);
  var areSetsEqual = createAreIterablesEqual(false);

  var createComparator = function createComparator(createIsEqual) {
    // eslint-disable-next-line no-use-before-define
    var isEqual = typeof createIsEqual === 'function' ? createIsEqual(comparator) : comparator;

    /**
     * @function comparator
     *
     * @description
     * compare the value of the two objects and return true if they are equivalent in values
     *
     * @param {any} objectA the object to test against
     * @param {any} objectB the object to test
     * @param {any} [meta] an optional meta object that is passed through to all equality test calls
     * @returns {boolean} are objectA and objectB equivalent in value
     */
    function comparator(objectA, objectB, meta) {
      if (sameValueZeroEqual(objectA, objectB)) {
        return true;
      }

      var typeOfA = typeof objectA;

      if (typeOfA !== typeof objectB || typeOfA !== 'object' || !objectA || !objectB) {
        return false;
      }

      if (isPlainObject(objectA) && isPlainObject(objectB)) {
        return areObjectsEqual(objectA, objectB, isEqual, meta);
      }

      var arrayA = isArray(objectA);
      var arrayB = isArray(objectB);

      if (arrayA || arrayB) {
        return arrayA === arrayB && areArraysEqual(objectA, objectB, isEqual, meta);
      }

      var dateA = objectA instanceof Date;
      var dateB = objectB instanceof Date;

      if (dateA || dateB) {
        return dateA === dateB && sameValueZeroEqual(objectA.getTime(), objectB.getTime());
      }

      var regexpA = objectA instanceof RegExp;
      var regexpB = objectB instanceof RegExp;

      if (regexpA || regexpB) {
        return regexpA === regexpB && areRegExpsEqual(objectA, objectB);
      }

      if (isPromiseLike(objectA) || isPromiseLike(objectB)) {
        return objectA === objectB;
      }

      if (HAS_MAP_SUPPORT) {
        var mapA = objectA instanceof Map;
        var mapB = objectB instanceof Map;

        if (mapA || mapB) {
          return mapA === mapB && areMapsEqual(objectA, objectB, isEqual, meta);
        }
      }

      if (HAS_SET_SUPPORT) {
        var setA = objectA instanceof Set;
        var setB = objectB instanceof Set;

        if (setA || setB) {
          return setA === setB && areSetsEqual(objectA, objectB, isEqual, meta);
        }
      }

      return areObjectsEqual(objectA, objectB, isEqual, meta);
    }

    return comparator;
  };

  // comparator

  var circularDeepEqual = createComparator(createCircularEqual());
  var circularShallowEqual = createComparator(createCircularEqual(sameValueZeroEqual));
  var deepEqual = createComparator();
  var shallowEqual = createComparator(function () {
    return sameValueZeroEqual;
  });

  /**
   * @function identity
   *
   * @description
   * return the first value passed
   *
   * @param {any} value the value to return
   * @returns {any} the first parameter passed
   */
  var identity = function identity(value) {
    return value;
  };
  /**
   * @function isUndefined
   *
   * @description
   * is the value passed undefined
   *
   * @param {any} value the value to test
   * @returns {boolean} is the value passed undefined
   */

  var isUndefined = function isUndefined(value) {
    return value === void 0;
  };
  /**
   * @function noop
   *
   * @description
   * return nothing regardless of what is passed
   *
   * @returns {void}
   */

  var noop = function noop() {};
  /**
   * @function reduce
   *
   * @description
   * reduce the array to a single value
   *
   * @param {Array<any>} array the array to reduce
   * @param {function} fn the function called with each iteration
   * @param {any} initialValue the initial value of the reduction
   * @returns {any} the reduced value
   */

  var reduce = function reduce(array, fn, initialValue) {
    var value = initialValue;

    for (var index = 0; index < array.length; index++) {
      value = fn(value, array[index], index, array);
    }

    return value;
  };
  /**
   * @function assign
   *
   * @description
   * shallowly merge the sources into the target object
   *
   * @param {Object} target the target object to merge into
   * @param  {...Object} sources the source objects to merge into target
   * @returns {Object} the merged object
   */

  var assign = function assign(target) {
    for (var _len = arguments.length, sources = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      sources[_key - 1] = arguments[_key];
    }

    return reduce(sources, function (assigned, source) {
      return typeof source === 'object' ? reduce(Object.keys(source), function (assignedSource, key) {
        assignedSource[key] = source[key];
        return assignedSource;
      }, assigned) : assigned;
    }, target);
  };
  /**
   * @function getActionCreators
   *
   * @description
   * get the actionCreators wrapped with the dispatch method
   *
   * @param {function|Object} actionCreators the actionCreators to wrap
   * @param {function} dispatch the instance-specific dispatch function
   * @returns {Object} the wrapped actionCreators
   */

  var getActionCreators = function getActionCreators(actionCreators, dispatch, ownProps) {
    return typeof actionCreators === 'function' ? actionCreators(dispatch, ownProps) : reduce(Object.keys(actionCreators), function (wrappedActionCreators, name) {
      wrappedActionCreators[name] = function () {
        return dispatch(actionCreators[name].apply(actionCreators, arguments));
      };

      return wrappedActionCreators;
    }, {});
  };
  /**
   * @function getFunctionNameRegexp
   *
   * @description
   * using regexp derivation of the function's toString value, get the function name
   *
   * @param {function} fn the function to derive the name from
   * @returns {string} the function name
   */

  var getFunctionNameRegexp = function getFunctionNameRegexp(fn) {
    return (/\W*function\s+([\w\$]+)\(/.exec("" + fn) || [])[1];
  };
  /**
   * @function getDisplayName
   *
   * @description
   * get the displayName of the component, either directly or derived
   *
   * @param {ReactComponent} Component the component to get the displayName for
   * @returns {string} the displayName
   */

  var getDisplayName = function getDisplayName(Component) {
    return Component.displayName || Component.name || getFunctionNameRegexp(Component) || 'Component';
  };
  /**
   * @function getMergedProps
   *
   * @description
   * get the state, actionCreators, and props merged into a single props object to pass
   *
   * @param {Object} stateProps the state of the reducer
   * @param {Object} dispatchProps the wrapped action creators
   * @param {Object} ownProps the props passed directly to the component
   * @returns {Object} the merged props
   */

  var getMergedProps = function getMergedProps(stateProps, dispatchProps, ownProps) {
    return assign({}, ownProps, stateProps, dispatchProps);
  };

  // external dependencies
  /**
   * @constant {Object} DEFAULT_OPTIONS
   */

  var DEFAULT_OPTIONS = {
    areMergedPropsEqual: shallowEqual,
    areOwnPropsEqual: shallowEqual,
    areStatesEqual: shallowEqual,
    mergeProps: getMergedProps,
    pure: true
  };

  var createGetState = function createGetState(instance) {
    return (
      /**
       * @function getState
       *
       * @description
       * get the current state of the component
       *
       * @returns {Object} the current state
       */
      function () {
        return instance.__state;
      }
    );
  };
  var createShouldComponentUpdate = function createShouldComponentUpdate(instance, actionCreators) {
    return (
      /**
       * @function shouldComponentUpdate
       *
       * @description
       * determine whether the component should update based on the options provided
       *
       * @param {Object} nextProps the next props of the component
       * @param {Object} nextState the next reducer values
       * @param {Object} nextContext the next context of the component
       * @returns {boolean} should the component update
       */
      function (nextProps, nextState, nextContext) {
        var __store = instance.__store,
            wrappedActionCreators = instance.actionCreators,
            context = instance.context,
            _instance$options = instance.options,
            areMergedPropsEqual = _instance$options.areMergedPropsEqual,
            areOwnPropsEqual = _instance$options.areOwnPropsEqual,
            areStatesEqual = _instance$options.areStatesEqual,
            mergeProps = _instance$options.mergeProps,
            pure = _instance$options.pure,
            props = instance.props,
            state = instance.state;

        if (typeof actionCreators === 'function' && (!pure || !areOwnPropsEqual(props, nextProps))) {
          instance.actionCreators = getActionCreators(actionCreators, __store.dispatch, nextProps);
        }

        if (!pure) {
          return true;
        }

        return !areOwnPropsEqual(props, nextProps) || !areStatesEqual(state, nextState) || !shallowEqual(context, nextContext) || !areMergedPropsEqual( // eslint workaround
        mergeProps(state, wrappedActionCreators, props), mergeProps(nextState, instance.actionCreators, nextProps));
      }
    );
  };
  var createComponentWillUnmount = function createComponentWillUnmount(instance) {
    return (
      /**
       * @function componentWillMount
       *
       * @description
       * on unmount, unsubscribe from the store and remove it
       */
      function () {
        instance.__unsubscribe();

        instance.__store = null;
      }
    );
  };
  /**
   * @function onConstruct
   *
   * @description
   * on construction of the instance, set the options and the instance-specific methods
   *
   * @param {ReactComponent} instance the component instance
   * @param {function} reducer the reducer with state
   * @param {Object} actionCreators the actionCreators to wrap with dispatch
   * @param {function} [enhancer=identity] the method to create the store enhancer
   * @param {Array<function>} [middlewares=[]] the list of middlewares to apply
   * @param {Object} options additional options for the component
   */

  var onConstruct = function onConstruct(instance, reducer, actionCreators, _temp) {
    var _ref = _temp === void 0 ? {} : _temp,
        _ref$enhancer = _ref.enhancer,
        enhancer = _ref$enhancer === void 0 ? identity : _ref$enhancer,
        _ref$middlewares = _ref.middlewares,
        middlewares = _ref$middlewares === void 0 ? [] : _ref$middlewares,
        options = _objectWithoutPropertiesLoose(_ref, ["enhancer", "middlewares"]);

    // options
    instance.options = assign({}, DEFAULT_OPTIONS, options); // store

    instance.__store = redux.createStore(reducer, enhancer(redux.applyMiddleware.apply(void 0, middlewares)));
    instance.__unsubscribe = instance.__store.subscribe(function () {
      return instance.setState(function () {
        return instance.__store.getState();
      });
    }); // lifecycle methods

    instance.shouldComponentUpdate = createShouldComponentUpdate(instance, actionCreators);
    instance.componentWillUnmount = createComponentWillUnmount(instance); // instance methods

    instance.getState = createGetState(instance); // instance values

    instance.actionCreators = actionCreators ? getActionCreators(actionCreators, instance.__store.dispatch, instance.props) : {
      dispatch: instance.__store.dispatch
    };
    instance.state = instance.__store.getState();
  };
  /**
   * @function connectLocal
   *
   * @description
   * decorate a component with a higher-order component that manages local state
   * in the way that redux manages global state
   *
   * @param {function} reducer the reducer with state
   * @param {Object} actionCreators the actionCreators to wrap with dispatch
   * @param {Object} options additional options for the component
   * @returns {function}
   */

  var connectLocal = function connectLocal(reducer, actionCreators, options) {
    return (
      /**
       * @function ConnectedLocalComponent
       *
       * @description
       * higher-order component decorator which adds a redux-like manager for local state
       *
       * @param {ReactComponent} ComponentToConnect the component being wrapped
       * @returns {ReactComponent} the higher-order component
       */
      function (ComponentToConnect) {
        function ConnectedLocalComponent(initialProps) {
          react.Component.call(this, initialProps);
          onConstruct(this, reducer, actionCreators, options);

          this.render = function () {
            return react.createElement(ComponentToConnect, this.options.mergeProps(this.state, this.actionCreators, this.props));
          };

          return this;
        }

        ConnectedLocalComponent.prototype = Object.create(react.Component.prototype);
        ConnectedLocalComponent.displayName = "ConnectedLocal(" + getDisplayName(ComponentToConnect) + ")";
        return ConnectedLocalComponent;
      }
    );
  };

  // utils
  /**
   * @function createAction
   *
   * @description
   * create an action creator based on the parameters passed
   *
   * @param {string} type the type of action being dispatched
   * @param {function} [payloadHandler=identity] the function to get the payload value
   * @param {function} [metaHandler=noop] the function to get the meta value
   * @returns {function(...any): Object} the actionCreator
   */

  var createActionCreator = function createActionCreator(type, payloadHandler, metaHandler) {
    if (payloadHandler === void 0) {
      payloadHandler = identity;
    }

    if (metaHandler === void 0) {
      metaHandler = noop;
    }

    if (typeof type !== 'string') {
      throw new ReferenceError('The action type passed must be a string.');
    }

    if (typeof payloadHandler !== 'function') {
      throw new ReferenceError('The payload handler passed must be a function.');
    }

    if (typeof metaHandler !== 'function') {
      throw new ReferenceError('The meta handler passed must be a function.');
    }

    var actionCreator = function actionCreator() {
      var action = {
        type: type
      };
      var payload = payloadHandler.apply(void 0, arguments);

      if (!isUndefined(payload)) {
        action.payload = payload;

        if (payload instanceof Error) {
          action.error = true;
        }
      }

      var meta = metaHandler.apply(void 0, arguments);

      if (!isUndefined(meta)) {
        action.meta = meta;
      }

      return action;
    };

    actionCreator.toString = function () {
      return type;
    };

    return actionCreator;
  };
  /**
   * @function createReducer
   *
   * @description
   * based on the handlers and initial state passed, build a reducer function
   *
   * @param {Object} handlers the handlers to build the reducer from
   * @param {Object} initialState the initial state of the reducer
   * @returns {function(Object, Object): Object} the reducer for the component
   */

  var createReducer = function createReducer(handlers, initialState) {
    if (initialState === void 0) {
      initialState = {};
    }

    if (typeof handlers === 'function') {
      return handlers;
    }

    if (typeof handlers !== 'object') {
      throw new ReferenceError('The handlers passed must be an object.');
    }

    return function (state, action) {
      if (state === void 0) {
        state = initialState;
      }

      var handler = handlers[action.type];
      return handler ? handler(state, action) : state;
    };
  };

  // decorators

  exports.connectLocal = connectLocal;
  exports.createActionCreator = createActionCreator;
  exports.createReducer = createReducer;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=react-local-redux.js.map
