function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

// external dependencies
import { shallowEqual } from 'fast-equals';
import { Component, createElement } from 'react';
import { applyMiddleware, createStore } from 'redux'; // constants

import { DEFAULT_OPTIONS } from './constants'; // utils

import { assign, getActionCreators, getDisplayName, identity } from './utils';
export var createGetState = function createGetState(instance) {
  return (
    /**
     * @function getState
     *
     * @description
     * get the current state of the component
     *
     * @returns {Object} the current state
     */
    function () {
      return instance.__state;
    }
  );
};
export var createShouldComponentUpdate = function createShouldComponentUpdate(instance, actionCreators) {
  return (
    /**
     * @function shouldComponentUpdate
     *
     * @description
     * determine whether the component should update based on the options provided
     *
     * @param {Object} nextProps the next props of the component
     * @param {Object} nextState the next reducer values
     * @param {Object} nextContext the next context of the component
     * @returns {boolean} should the component update
     */
    function (nextProps, nextState, nextContext) {
      var __store = instance.__store,
          wrappedActionCreators = instance.actionCreators,
          context = instance.context,
          _instance$options = instance.options,
          areMergedPropsEqual = _instance$options.areMergedPropsEqual,
          areOwnPropsEqual = _instance$options.areOwnPropsEqual,
          areStatesEqual = _instance$options.areStatesEqual,
          mergeProps = _instance$options.mergeProps,
          pure = _instance$options.pure,
          props = instance.props,
          state = instance.state;

      if (typeof actionCreators === 'function' && (!pure || !areOwnPropsEqual(props, nextProps))) {
        instance.actionCreators = getActionCreators(actionCreators, __store.dispatch, nextProps);
      }

      if (!pure) {
        return true;
      }

      return !areOwnPropsEqual(props, nextProps) || !areStatesEqual(state, nextState) || !shallowEqual(context, nextContext) || !areMergedPropsEqual( // eslint workaround
      mergeProps(state, wrappedActionCreators, props), mergeProps(nextState, instance.actionCreators, nextProps));
    }
  );
};
export var createComponentWillUnmount = function createComponentWillUnmount(instance) {
  return (
    /**
     * @function componentWillMount
     *
     * @description
     * on unmount, unsubscribe from the store and remove it
     */
    function () {
      instance.__unsubscribe();

      instance.__store = null;
    }
  );
};
/**
 * @function onConstruct
 *
 * @description
 * on construction of the instance, set the options and the instance-specific methods
 *
 * @param {ReactComponent} instance the component instance
 * @param {function} reducer the reducer with state
 * @param {Object} actionCreators the actionCreators to wrap with dispatch
 * @param {function} [enhancer=identity] the method to create the store enhancer
 * @param {Array<function>} [middlewares=[]] the list of middlewares to apply
 * @param {Object} options additional options for the component
 */

export var onConstruct = function onConstruct(instance, reducer, actionCreators, _temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      _ref$enhancer = _ref.enhancer,
      enhancer = _ref$enhancer === void 0 ? identity : _ref$enhancer,
      _ref$middlewares = _ref.middlewares,
      middlewares = _ref$middlewares === void 0 ? [] : _ref$middlewares,
      options = _objectWithoutPropertiesLoose(_ref, ["enhancer", "middlewares"]);

  // options
  instance.options = assign({}, DEFAULT_OPTIONS, options); // store

  instance.__store = createStore(reducer, enhancer(applyMiddleware.apply(void 0, middlewares)));
  instance.__unsubscribe = instance.__store.subscribe(function () {
    return instance.setState(function () {
      return instance.__store.getState();
    });
  }); // lifecycle methods

  instance.shouldComponentUpdate = createShouldComponentUpdate(instance, actionCreators);
  instance.componentWillUnmount = createComponentWillUnmount(instance); // instance methods

  instance.getState = createGetState(instance); // instance values

  instance.actionCreators = actionCreators ? getActionCreators(actionCreators, instance.__store.dispatch, instance.props) : {
    dispatch: instance.__store.dispatch
  };
  instance.state = instance.__store.getState();
};
/**
 * @function connectLocal
 *
 * @description
 * decorate a component with a higher-order component that manages local state
 * in the way that redux manages global state
 *
 * @param {function} reducer the reducer with state
 * @param {Object} actionCreators the actionCreators to wrap with dispatch
 * @param {Object} options additional options for the component
 * @returns {function}
 */

export var connectLocal = function connectLocal(reducer, actionCreators, options) {
  return (
    /**
     * @function ConnectedLocalComponent
     *
     * @description
     * higher-order component decorator which adds a redux-like manager for local state
     *
     * @param {ReactComponent} ComponentToConnect the component being wrapped
     * @returns {ReactComponent} the higher-order component
     */
    function (ComponentToConnect) {
      function ConnectedLocalComponent(initialProps) {
        Component.call(this, initialProps);
        onConstruct(this, reducer, actionCreators, options);

        this.render = function () {
          return createElement(ComponentToConnect, this.options.mergeProps(this.state, this.actionCreators, this.props));
        };

        return this;
      }

      ConnectedLocalComponent.prototype = Object.create(Component.prototype);
      ConnectedLocalComponent.displayName = "ConnectedLocal(" + getDisplayName(ComponentToConnect) + ")";
      return ConnectedLocalComponent;
    }
  );
};