/**
 * @function identity
 *
 * @description
 * return the first value passed
 *
 * @param {any} value the value to return
 * @returns {any} the first parameter passed
 */
export var identity = function identity(value) {
  return value;
};
/**
 * @function isUndefined
 *
 * @description
 * is the value passed undefined
 *
 * @param {any} value the value to test
 * @returns {boolean} is the value passed undefined
 */

export var isUndefined = function isUndefined(value) {
  return value === void 0;
};
/**
 * @function noop
 *
 * @description
 * return nothing regardless of what is passed
 *
 * @returns {void}
 */

export var noop = function noop() {};
/**
 * @function reduce
 *
 * @description
 * reduce the array to a single value
 *
 * @param {Array<any>} array the array to reduce
 * @param {function} fn the function called with each iteration
 * @param {any} initialValue the initial value of the reduction
 * @returns {any} the reduced value
 */

export var reduce = function reduce(array, fn, initialValue) {
  var value = initialValue;

  for (var index = 0; index < array.length; index++) {
    value = fn(value, array[index], index, array);
  }

  return value;
};
/**
 * @function assign
 *
 * @description
 * shallowly merge the sources into the target object
 *
 * @param {Object} target the target object to merge into
 * @param  {...Object} sources the source objects to merge into target
 * @returns {Object} the merged object
 */

export var assign = function assign(target) {
  for (var _len = arguments.length, sources = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    sources[_key - 1] = arguments[_key];
  }

  return reduce(sources, function (assigned, source) {
    return typeof source === 'object' ? reduce(Object.keys(source), function (assignedSource, key) {
      assignedSource[key] = source[key];
      return assignedSource;
    }, assigned) : assigned;
  }, target);
};
/**
 * @function getActionCreators
 *
 * @description
 * get the actionCreators wrapped with the dispatch method
 *
 * @param {function|Object} actionCreators the actionCreators to wrap
 * @param {function} dispatch the instance-specific dispatch function
 * @returns {Object} the wrapped actionCreators
 */

export var getActionCreators = function getActionCreators(actionCreators, dispatch, ownProps) {
  return typeof actionCreators === 'function' ? actionCreators(dispatch, ownProps) : reduce(Object.keys(actionCreators), function (wrappedActionCreators, name) {
    wrappedActionCreators[name] = function () {
      return dispatch(actionCreators[name].apply(actionCreators, arguments));
    };

    return wrappedActionCreators;
  }, {});
};
/**
 * @function getFunctionNameRegexp
 *
 * @description
 * using regexp derivation of the function's toString value, get the function name
 *
 * @param {function} fn the function to derive the name from
 * @returns {string} the function name
 */

export var getFunctionNameRegexp = function getFunctionNameRegexp(fn) {
  return (/\W*function\s+([\w\$]+)\(/.exec("" + fn) || [])[1];
};
/**
 * @function getDisplayName
 *
 * @description
 * get the displayName of the component, either directly or derived
 *
 * @param {ReactComponent} Component the component to get the displayName for
 * @returns {string} the displayName
 */

export var getDisplayName = function getDisplayName(Component) {
  return Component.displayName || Component.name || getFunctionNameRegexp(Component) || 'Component';
};
/**
 * @function getMergedProps
 *
 * @description
 * get the state, actionCreators, and props merged into a single props object to pass
 *
 * @param {Object} stateProps the state of the reducer
 * @param {Object} dispatchProps the wrapped action creators
 * @param {Object} ownProps the props passed directly to the component
 * @returns {Object} the merged props
 */

export var getMergedProps = function getMergedProps(stateProps, dispatchProps, ownProps) {
  return assign({}, ownProps, stateProps, dispatchProps);
};