"use strict";

exports.__esModule = true;
exports.createReducer = exports.createActionCreator = void 0;

var _utils = require("./utils");

// utils

/**
 * @function createAction
 *
 * @description
 * create an action creator based on the parameters passed
 *
 * @param {string} type the type of action being dispatched
 * @param {function} [payloadHandler=identity] the function to get the payload value
 * @param {function} [metaHandler=noop] the function to get the meta value
 * @returns {function(...any): Object} the actionCreator
 */
var createActionCreator = function createActionCreator(type, payloadHandler, metaHandler) {
  if (payloadHandler === void 0) {
    payloadHandler = _utils.identity;
  }

  if (metaHandler === void 0) {
    metaHandler = _utils.noop;
  }

  if (typeof type !== 'string') {
    throw new ReferenceError('The action type passed must be a string.');
  }

  if (typeof payloadHandler !== 'function') {
    throw new ReferenceError('The payload handler passed must be a function.');
  }

  if (typeof metaHandler !== 'function') {
    throw new ReferenceError('The meta handler passed must be a function.');
  }

  var actionCreator = function actionCreator() {
    var action = {
      type: type
    };
    var payload = payloadHandler.apply(void 0, arguments);

    if (!(0, _utils.isUndefined)(payload)) {
      action.payload = payload;

      if (payload instanceof Error) {
        action.error = true;
      }
    }

    var meta = metaHandler.apply(void 0, arguments);

    if (!(0, _utils.isUndefined)(meta)) {
      action.meta = meta;
    }

    return action;
  };

  actionCreator.toString = function () {
    return type;
  };

  return actionCreator;
};
/**
 * @function createReducer
 *
 * @description
 * based on the handlers and initial state passed, build a reducer function
 *
 * @param {Object} handlers the handlers to build the reducer from
 * @param {Object} initialState the initial state of the reducer
 * @returns {function(Object, Object): Object} the reducer for the component
 */


exports.createActionCreator = createActionCreator;

var createReducer = function createReducer(handlers, initialState) {
  if (initialState === void 0) {
    initialState = {};
  }

  if (typeof handlers === 'function') {
    return handlers;
  }

  if (typeof handlers !== 'object') {
    throw new ReferenceError('The handlers passed must be an object.');
  }

  return function (state, action) {
    if (state === void 0) {
      state = initialState;
    }

    var handler = handlers[action.type];
    return handler ? handler(state, action) : state;
  };
};

exports.createReducer = createReducer;